Package: date4ts
Title: Wrangle and Modify Ts Object with Classic Frequencies and Exact
        Dates
Version: 0.1.2
Authors@R: 
    person("Tanguy", "Barthelemy", email = "tangbarth@hotmail.fr", role = c("aut", "cre"))
Description: The ts objects in R are managed using a very specific date format (in the form c(2022, 9) for September 2022 or c(2021, 2) for the second quarter of 2021, depending on the frequency, for example). We focus solely on monthly and quarterly series to manage the dates of ts objects. The general idea is to offer a set of functions to manage this date format without it being too restrictive or too imprecise depending on the rounding. This is a compromise between simplicity, precision and use of the basic 'stats' functions for creating and managing time series (ts(), window()).
    Les objets ts en R sont gérés par un format de date très particulier (sous la forme c(2022, 9) pour septembre 2022 ou c(2021, 2) pour le deuxième trimestre 2021 selon la fréquence par exemple). On se concentre uniquement sur les séries mensuelles et trimestrielles pour gérer les dates des objets ts. Lidée générale est de proposer un ensemble de fonctions pour gérer ce format de date sans que ce soit trop contraignant ou trop imprécis selon les arrondis. Cest un compromis entre simplicité, précision et utilisation des fonctions du package 'stats' de création et de gestion des séries temporelles (ts(), window()).
License: GPL (>= 3)
URL: https://github.com/TractorTom/date4ts,
        https://tractortom.github.io/date4ts/
BugReports: https://github.com/TractorTom/date4ts/issues
Encoding: UTF-8
Language: fr-FR
RoxygenNote: 7.3.3
Imports: stats, checkmate
Depends: R (>= 4.1)
LazyData: true
Suggests: testthat (>= 3.0.0), renv, fuzzr, pkgdown, devtools, usethis,
        covr, withr, altdoc
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 14:07:09 UTC; onyxia
Author: Tanguy Barthelemy [aut, cre]
Maintainer: Tanguy Barthelemy <tangbarth@hotmail.fr>
Repository: CRAN
Date/Publication: 2026-04-29 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 15:57:17 UTC; unix
