# Generated by roxygen2: do not edit by hand

export(as_yyyymm)
export(as_yyyytt)
export(assert_date_ts)
export(assert_expression)
export(assert_frequency)
export(assert_scalar_date)
export(assert_scalar_integer)
export(assert_scalar_natural)
export(assert_timeunits)
export(assert_ts)
export(check_date_ts)
export(check_expression)
export(check_frequency)
export(check_scalar_date)
export(check_scalar_integer)
export(check_scalar_natural)
export(check_timeunits)
export(check_ts)
export(combine2ts)
export(date2date_ts)
export(date_ts2date)
export(date_ts2timeunits)
export(diff_periode)
export(extend_ts)
export(first_date)
export(get_value_ts)
export(is_before)
export(last_date)
export(libelles)
export(mens2trim)
export(na_trim)
export(next_date_ts)
export(normalize_date_ts)
export(previous_date_ts)
export(set_value_ts)
export(substr_year)
export(trim2mens)
export(ts2df)
importFrom(stats,frequency)
